
/*
 * Copyright 1998 by PsychoticSoftware
 *
 * This program is free software; you can redistribute it and/or
 * modify it. All we ask is for a simple dredit in any application 
 * you use any part of this source code in -- maybe in a README file.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 */


package SuperEdit;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.accessibility.*;
import com.sun.java.swing.*;
import com.sun.java.swing.undo.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.text.*;
import java.io.*;
import com.sun.java.swing.tree.*;

public class Editor extends JInternalFrame
{
  //Browser.TreePanel topPanel =  new Browser.TreePanel(getProjectRoot());
  ClassNode structure =  new ClassNode(new TE("Editor.java"));

  JPanel jPanel1 = new STextPanel(this);
  BorderLayout borderLayout1 = new BorderLayout();
  Workspace par = null;

  private boolean hasLanguagePackageInstalled = true;

  JToolBar toolBar = new JToolBar();
  public DefaultStyledDocument doc;
  public JTextComponent theText;
  public boolean wordFound = false;
  public int x = 0;
  public int y = 0;
  public Fontifier theListener;
  public Thread watcher;
  public Style currentStyle;
  String retBuffer = "";
  JComponent status;
  private JInternalFrame frame;
  File currentFileName = null;
  protected MyTimer autoSaver = null;
  private Object tmpHighlight = null;

  EditorOptions opts;

  UndoAction undoAction;
  RedoAction redoAction;

  JButton newButton;
  JButton openButton;
  JButton saveButton;
  JButton cutButton;
  JButton copyButton;
  JButton pasteButton;
  JButton undoButton;
  JButton redoButton;
  JButton optionsButton;
  JButton fontifyButton;
  JButton structureButton;

  JComponent verticalRule;
  //SuperEdit superEdit;

  public Editor(Workspace p)
  {
    try
    {
      this.par = p;
      frame = this;
      opts = new EditorOptions(this);
      jbInit();
      watcher = new Looper(this);
      watcher.start();
      watcher.suspend();
      if(SuperEdit.superEdit.ideOptions.enableAutosave.isSelected())
      {
        autoSaver = new MyTimer(((Integer)SuperEdit.superEdit.ideOptions.timeCombo.getSelectedItem()).intValue());
        autoSaver.start();
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  public Editor(Workspace p, File f)
  {
    this(p);
    currentFileName = f;
    setTitle(f.getAbsolutePath());
    Thread fl = new FileLoader(f, doc);
    fl.start();
  }

  /*public void setStyles(StyleContext styles)
  {
    this.styles = styles;
  }*/

  public class STextPanel extends JPanel
  {
     public WrapableTextPane text = initDoc();

     public STextPanel(Editor ed)
     {
      //text.setFont(new Font("Courier", Font.PLAIN, 14));
      theText = text;
      Fontifier listener =  new Fontifier(ed);
      text.addCaretListener(listener);
      theListener = listener;
      setLayout(new BorderLayout());
      add(new EditScroller( text ), BorderLayout.CENTER);
     }
  }

  public void addFontifier(Fontifier f)
  {
    this.theText.addCaretListener(f);
    hasLanguagePackageInstalled =  true;
  }

  public void removeFontifier(Fontifier f)
  {
    this.theText.removeCaretListener(f);
    hasLanguagePackageInstalled = false;
  }

  WrapableTextPane initDoc()
  {

    StyleContext styles = new StyleContext();
    doc = new DefaultStyledDocument(styles);
    return new WrapableTextPane(doc, false);
  }

  class WrapableTextPane extends JTextPane
  {
   boolean wrap = false;

   public WrapableTextPane(DefaultStyledDocument comp, boolean wrap)
   {
    super(comp);
    this.wrap = wrap;
   }

   public boolean getScrollableTracksViewportWidth()
   {
    return wrap;
   }

   public Dimension getPreferredScrollableViewportSize()
   {
     return new Dimension(400, 400);
   }

   public void processMouseEvent(java.awt.event.MouseEvent me)
   {
     if(me.isPopupTrigger())
     {
       pop.show(this, me.getX(), me.getY());
     }
     else
     {
       super.processMouseEvent(me);
     }
   }
  }

  public class EditScroller extends JScrollPane
  {
     private JLabel makeLabel(String name, String description)
     {
       String filename = "images/" + name;
       java.net.URL url = getClass().getResource(filename);
	     ImageIcon image = new ImageIcon(url, description);
	     return new JLabel(image);
     }

     public EditScroller(WrapableTextPane theText)
     {
       super(theText);
       if(SuperEdit.superEdit.ideOptions.showRulerBox.isSelected())
       {
         verticalRule = makeLabel("column.gif", "Vertical ruler carved out of stone");
	       verticalRule.getAccessibleContext().setAccessibleName("Vertical rule");
         setRowHeaderView(verticalRule);
       }
     }


     public boolean isOpaque()
     {
       return true;
     }
  }

  public ImageIcon loadImageIcon(String filename, String description)
  {
      java.net.URL url = getClass().getResource(filename);
      return new ImageIcon(url, description);
  }

  public JButton addTool(JToolBar toolBar, String name)
  {
	  JButton b = (JButton) toolBar.add(
                new JButton(loadImageIcon("images/" + name + ".gif",name)));
	  b.setToolTipText(name);
	  b.setMargin(new Insets(0,0,0,0));
	  b.getAccessibleContext().setAccessibleName(name);
    return b;
  }

  private void jbInit() throws Exception
  {
    ClassNode t2 = new ClassNode(new TE("Second Inner"));
    structure.addInnerClass(t2);

    this.toolBar.setFloatable(true);
	  this.newButton = addTool(toolBar, "new");
	  this.openButton = addTool(toolBar, "open");
	  this.saveButton = addTool(toolBar, "save");
	  this.toolBar.addSeparator();
	  this.cutButton = addTool(toolBar, "cut");
	  this.copyButton = addTool(toolBar, "copy");
	  this.pasteButton = addTool(toolBar, "paste");
	  this.toolBar.addSeparator();
	  this.undoButton = addTool(toolBar, "undo");
	  this.redoButton = addTool(toolBar, "redo");
	  this.toolBar.addSeparator();
	  this.optionsButton = addTool(toolBar, "options");
	  this.fontifyButton = addTool(toolBar, "fontify");
	  this.structureButton = addTool(toolBar, "structure");

    this.getContentPane().setLayout(borderLayout1);
    this.setClosable(true);
    this.setResizable(true);
    this.setTitle("<Untitled>");
    this.setMaximizable(true);
    this.setIconifiable(true);
    theText.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
        theText_mouseClicked(e);
      }
    });
    newButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        newButton_actionPerformed(e);
      }
    });
    structureButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        structureButton_actionPerformed(e);
      }
    });
    theText.addFocusListener(new java.awt.event.FocusAdapter()
    {
      public void focusGained(FocusEvent e)
      {
        theText_focusGained(e);
      }
    });
    theText.addKeyListener(new java.awt.event.KeyAdapter()
    {
      public void keyTyped(KeyEvent e)
      {
        theText_keyTyped(e);
      }
    });
    saveButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        saveButton_actionPerformed(e);
      }
    });
    /*
    theText.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mousePressed(MouseEvent e)
      {
        theText_mousePressed(e);
      }
    });
    */
    removeHighlightPopup.setText("-Highlight");
    removeHighlightPopup.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        removeHighlightPopup_actionPerformed(e);
      }
    });
    //toolsPopup.setText("Tools");
    highlightPopup.setText("+Highlight");
    //highlightPopup.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));

    highlightPopup.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        highlightPopup_actionPerformed(e);
      }
    });
    this.addInternalFrameListener(new com.sun.java.swing.event.InternalFrameAdapter()
    {
      public void internalFrameClosing(InternalFrameEvent e)
      {
        this_internalFrameClosing(e);
      }
    });
    fontifyButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        fontifyButton_actionPerformed(e);
      }
    });
    openButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        openButton_actionPerformed(e);
      }
    });
    optionsButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        optionsButton_actionPerformed(e);
      }
    });
    pasteButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        pasteButton_actionPerformed(e);
      }
    });
    copyButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        copyButton_actionPerformed(e);
      }
    });
    cutButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        cutButton_actionPerformed(e);
      }
    });


    this.getContentPane().add(jPanel1, BorderLayout.CENTER);

    this.getContentPane().add(toolBar, BorderLayout.NORTH);
    this.setBounds(par.getSize().width/2 - 230, par.getSize().height/2 - 240, 460, 480);

    undoAction = new UndoAction();
    redoAction = new RedoAction();

    undoButton.addActionListener(undoAction);
    redoButton.addActionListener(redoAction);
    doc.addUndoableEditListener(undoHandler);

    this.getContentPane().add(createStatusbar(), BorderLayout.SOUTH);
    pop.add(highlightPopup);
    pop.add(removeHighlightPopup);
    //toolsPopup.add(highlightPopup);

    highlightPopup.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));
    removeHighlightPopup.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.ALT_MASK));

    /*
       Button Accelerators
       pasteButton.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
    */
    saveButton.setEnabled(false);

  }

  protected UndoableEditListener undoHandler = new UndoHandler();
  protected UndoManager undo = new UndoManager();
  JPopupMenu pop = new JPopupMenu();
  JMenuItem removeHighlightPopup = new JMenuItem();
  //JMenu toolsPopup = new JMenu();
  JMenuItem highlightPopup = new JMenuItem();

  class UndoHandler implements UndoableEditListener{
    public void undoableEditHappened(UndoableEditEvent e)
    {
	    undo.addEdit(e.getEdit());
	    undoAction.update();
	    redoAction.update();
    }
  }

  class UndoAction extends AbstractAction
  {
	  public UndoAction()
    {
	    super("Undo");
	    undoButton.setEnabled(false);
    }

	  public void actionPerformed(ActionEvent e)
    {
	    try
      {
		    undo.undo();
	    }
      catch (CannotUndoException ex)
      {
		    System.out.println("Unable to undo: " + ex);
		    ex.printStackTrace();
	    }
	    update();
	    redoAction.update();
    }

	  protected void update()
    {
	    if(undo.canUndo())
      {
		    undoButton.setEnabled(true);
		    putValue(Action.NAME, undo.getUndoPresentationName());
	    }
	    else
      {
		    undoButton.setEnabled(false);
		    putValue(Action.NAME, "Undo");
	    }
        par.parent.undoItem.setEnabled(undoButton.isEnabled());
        par.parent.redoItem.setEnabled(redoButton.isEnabled());
    }
  }

  class RedoAction extends AbstractAction
  {
	  public RedoAction()
    {
	    super("Redo");
	    redoButton.setEnabled(false);
    }

	  public void actionPerformed(ActionEvent e)
    {
	    try
      {
		    undo.redo();
	    }
      catch (CannotRedoException ex)
      {
		    System.out.println("Unable to redo: " + ex);
		    ex.printStackTrace();
	    }
	    update();
	    undoAction.update();
    }

	  protected void update()
    {
	    if(undo.canRedo())
      {
		    redoButton.setEnabled(true);
		    putValue(Action.NAME, undo.getRedoPresentationName());
	    }
	    else
      {
		    redoButton.setEnabled(false);
		    putValue(Action.NAME, "Redo");
	    }
        par.parent.undoItem.setEnabled(undoButton.isEnabled());
        par.parent.redoItem.setEnabled(redoButton.isEnabled());
    }
  }

  protected JComponent createStatusbar()
  {
	  // need to do something reasonable here
	  status = new StatusBar();
    return status;
  }

  class StatusBar extends JComponent
  {
    public StatusBar()
    {
	    super();
	    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
    }

    public void paint(Graphics g)
    {
	    super.paint(g);
	  }
  }

  class FileLoader extends Thread
  {

	  FileLoader(File f, Document d)
    {
      setPriority(4);
      this.d = d;
      this.f = f;
    }

    public void run()
    {
	    try
      {
        //if(d==null) {System.err.println("Document is null");}
        d.removeUndoableEditListener(undoHandler);
        removeFontifier(theListener);
		    // initialize the statusbar
		    status.removeAll();
		    JProgressBar progress = new JProgressBar();
		    progress.setMinimum(0);
		    progress.setMaximum((int) f.length());
		    status.add(progress);
        if(par.parent.ideOptions.showProgressBox.isSelected())
        {
		      status.revalidate();
        }
		    // try to start reading
		    BufferedReader in = new BufferedReader(new FileReader(f));
		    char[] buff = new char[4096];
		    int nch;
		    while ((nch = in.read(buff, 0, buff.length)) != -1)
        {
		      d.insertString(d.getLength(), new String(buff, 0, nch), null);
		      progress.setValue(progress.getValue() + nch);
        }

		    // we are done... get rid of progressbar
        in.close();
		    d.addUndoableEditListener(undoHandler);
        addFontifier(theListener);
		    status.removeAll();
        if(par.parent.ideOptions.showProgressBox.isSelected())
        {
		      status.revalidate();
        }
	    }
      catch (Exception ex)
      {
		    //System.out.println("A thread race condition has been detected.");
		    //System.out.println("Trying to reload file.");
		    //System.err.println("A thread race condition has been detected.");
        //ex.printStackTrace();
        theText.setText("");
        theText.repaint();
        (new FileLoader(f, d)).start();
      }
    }

	  Document d;
	  File f;
  }


  void cutButton_actionPerformed(ActionEvent e)
  {
    theText.cut();
  }

  void copyButton_actionPerformed(ActionEvent e)
  {
    theText.copy();
  }

  void pasteButton_actionPerformed(ActionEvent e)
  {
    theText.paste();
  }

  void optionsButton_actionPerformed(ActionEvent e)
  {
    opts.setVisible(true);
  }

  void openButton_actionPerformed(ActionEvent e)
  {
    Chooser chooser = new Chooser();
    chooser.setOpen();
    chooser.setTitle("Open File");

	  int retval = chooser.showDialog(this, null);
	  if(retval == Chooser.APPROVE_OPTION)
    {
	    File theFile = chooser.getSelectedFile();
	    if(theFile != null)
      {
		    if(theFile.exists())
        {
		      //JOptionPane.showMessageDialog(this, "You chose this file: " + chooser.getSelectedFile().getAbsolutePath());
          currentFileName = theFile;
          frame.setTitle(theFile.getAbsolutePath());
          Thread fl = new FileLoader(theFile, doc);
          fl.start();
        }
        else
        {
          JOptionPane.showMessageDialog(this, "File does not exist: " + chooser.getSelectedFile().getAbsolutePath());
        }
		    return;
      }
    }
  JOptionPane.showMessageDialog(this, "No file was chosen.");
  }

  void fontifyButton_actionPerformed(ActionEvent e)
  {
    if(!hasLanguagePackageInstalled) {return;}
    (new Scan(this)).start();
  }

  void this_internalFrameClosing(InternalFrameEvent e)
  {
     if(autoSaver != null)
     {
       autoSaver.stop();
       autoSaver = null;
     }

    // Ask user to save unsaved unsaved files
    if(saveButton.isEnabled())
    {
      int res =  JOptionPane.showConfirmDialog(this, "Save " + getTitle() + "?");
      if(res == JOptionPane.OK_OPTION)
      {
        saveButton_actionPerformed(null);
      }

    }

    // Remove this from the list of opened windows.
    par.parent.openedWindows.removeElement(this);
    dispose();
  }

  void theText_mousePressed(MouseEvent e)
  {

  }

  void highlightPopup_actionPerformed(ActionEvent e)
  {
    light(null);
  }

  Object light(Color C)
  {
    Object res = null;

    Highlighter h = theText.getHighlighter();
    Highlighter.HighlightPainter hp = new DefaultHighlighter.DefaultHighlightPainter((C==null) ? SuperEdit.superEdit.ideOptions.highlightColor : C);

    Point info = getLineInfo(theText.getCaretPosition());

    try
    {
      res = h.addHighlight(info.x, info.y, hp);
      //h.addHighlight(theText.getCaretPosition()-5, theText.getCaretPosition()+5, hp);
    }
    catch(Exception he){System.out.println("Couldn't add highlight");}

    return res;
  }

  void dark(Object C)
  {
    Highlighter h = theText.getHighlighter();

    try
    {
      h.removeHighlight(C);
    }
    catch(Exception x){;}
  }


  public Point getLineInfo(int off)
  {
    int x = off-1;
    int y = off+1;

    try
    {
      while(!doc.getText(x, 1).equals("\n"))
      {
        x -= 1;
      }
    }
    catch(Exception e1) {x = 0;}

    try
    {
      while(!doc.getText(y, 1).equals("\n"))
      {
        y += 1;
      }
    }
    catch(Exception e1) {y = doc.getLength();}

    return new Point( x==0 ? x : x+1, y==doc.getLength() ? y : y+1);
  }


  void saveButton_actionPerformed(ActionEvent e)
  {
    if(currentFileName == null)
    {
      Chooser chooser = new Chooser();
      chooser.setSave();
      chooser.setTitle("Save File -> " + getTitle());
      File theFile;

	    int retval = chooser.showDialog(this, null);
	    if(retval == Chooser.APPROVE_OPTION)
      {
	      theFile = chooser.getSelectedFile();
	      if(theFile != null)
        {
		      if(theFile.exists())
          {
            int retval2 = JOptionPane.showConfirmDialog(this, "File already exists. Overwrite?");
            if(retval2 != JOptionPane.OK_OPTION)
            {
              return;
            }
          }
          try
          {
            FileOutputStream fos = new FileOutputStream(theFile);
            DataOutputStream dos = new DataOutputStream(fos);
            dos.writeBytes(doc.getText(0, doc.getLength()));
            currentFileName = theFile;
            frame.setTitle(theFile.getAbsolutePath());
            saveButton.setEnabled(false);
            dos.close();
            fos.close();
            SuperEdit.superEdit.mainStatus.setText("Saved " + currentFileName.getName() + " at " + new java.util.Date());
            SuperEdit.superEdit.mainStatus.repaint();
          }
          catch(Exception ee3)
          {
            JOptionPane.showMessageDialog(this, "Could not write file");
          }
		      return;
        }
      }
    }
    else
    {
      try
      {
        if(SuperEdit.superEdit.ideOptions.backupBox.isSelected())
        {
          currentFileName.renameTo(new File(currentFileName.getAbsolutePath() + ".bak"));
        }

        FileOutputStream fos = new FileOutputStream(currentFileName);
        DataOutputStream dos = new DataOutputStream(fos);
        dos.writeBytes(doc.getText(0, doc.getLength()));
        saveButton.setEnabled(false);
        dos.close();
        fos.close();
        SuperEdit.superEdit.mainStatus.setText("Saved " + currentFileName.getName() + " at " + new java.util.Date());
        SuperEdit.superEdit.mainStatus.repaint();
      }
      catch(Exception ee3)
      {
        JOptionPane.showMessageDialog(this, "Could not write file");
      }
    }
  }

  void theText_keyTyped(KeyEvent e)
  {
    saveButton.setEnabled(true);

    if(tmpHighlight != null)
    {
      dark(tmpHighlight);
      tmpHighlight = null;
    }

    if(SuperEdit.superEdit.ideOptions.autoIndentBox.isSelected())
    {
      boolean seenBrace = false;
      boolean seenOther = false;
      
      try
      {
        if(e.getKeyChar() == '\n' || e.getKeyChar() == '\r')
        {
          int start = theText.getCaretPosition() - 2;

          while(start > 0 && !(theText.getText(start, 1).equals("\n") || theText.getText(start, 1).equals("\r")))
          {
            if(theText.getText(start, 1).equals("{")) {seenBrace = true;}
            else if(!(theText.getText(start, 1).equals(" ") || theText.getText(start, 1).equals("\t"))) {seenOther = true;}

            start -= 1;
          }

          if(start > 0) {start += 1;}

          while(theText.getText(start, 1).equals(" ") || theText.getText(start, 1).equals("\t"))
          {
            retBuffer += theText.getText(start, 1);
            start += 1;
          }

          StyleContext styles = new StyleContext();
          Style def = styles.getStyle(StyleContext.DEFAULT_STYLE);
          Style normal = styles.addStyle("normal", def);
          StyleConstants.setFontFamily(normal, (String)SuperEdit.superEdit.ideOptions.fFamilyCombo.getSelectedItem());
          StyleConstants.setFontSize(normal, ((Integer)SuperEdit.superEdit.ideOptions.fSizeCombo.getSelectedItem()).intValue());
          StyleConstants.setForeground(normal, SuperEdit.superEdit.ideOptions.defaultTextColor);
          StyleConstants.setBold(normal, false);
          StyleConstants.setItalic(normal, false);


          doc.insertString(theText.getCaretPosition(), retBuffer, normal);

          if(SuperEdit.superEdit.ideOptions.indentBox.isSelected())
          {
            if(seenBrace && !seenOther)
            {
              doc.insertString(theText.getCaretPosition(),
              genWhite(((Integer)SuperEdit.superEdit.ideOptions.indentCombo.getSelectedItem()).intValue()),
              normal);
            }
          }
          retBuffer = "";
        }
      }
      catch(Exception v){;}
    }
  }

  private String genWhite(int sz)
  {
    if(sz <= 0) {return "";}
    else {return " " + genWhite(sz - 1);}
  }

  public DefaultMutableTreeNode getProjectRoot()
  {
   DefaultMutableTreeNode root = new DefaultMutableTreeNode("Untitled.spj");
   root.add(new DefaultMutableTreeNode("Untitled.java"));

   return root;
  }

  public String toString()
  {
    return (currentFileName!=null) ? currentFileName.getName() : this.getTitle();
  }

  void theText_focusGained(FocusEvent e)
  {
    par.parent.currentEditor = this;

    par.parent.cutItem.setEnabled(true);
    par.parent.copyItem.setEnabled(true);
    par.parent.pasteItem.setEnabled(true);
    par.parent.undoItem.setEnabled(undoButton.isEnabled());
    par.parent.redoItem.setEnabled(redoButton.isEnabled());

    //System.out.println(this + " has the focus.");

    //this.par.browser.updateBottom(this.bottomPanel);
    //this.par.browser.bottomPanel.update(structure);
    //this.par.browser.bottomPanel.model.reload();
    //this.par.browser.bottomPanel.repaint();
  }

  void structureButton_actionPerformed(ActionEvent e)
  {
    if(!hasLanguagePackageInstalled) {return;}
    (new StructureScanner(this)).start();
  }

  public void gotoLine(int line)
  {
    if(tmpHighlight != null)
    {
      dark(tmpHighlight);
      tmpHighlight = null;
    }

    int gotoL = -1;

    String data = theText.getText();
    if(line > data.length()) {return;}

    try
    {
      for(int i = 0; i < line - 1; i++)
      {
        int res;
        res = data.indexOf("\n", gotoL + 1);
        if(res < 0) {res = data.indexOf("\r", gotoL + 1);}
        gotoL = res;
      }
    }
    catch(Exception x) {gotoL = 0;}

    theText.setCaretPosition(gotoL <= 0 ? 0 : gotoL + 1);
    tmpHighlight = light(Color.red);
  }

  void newButton_actionPerformed(ActionEvent e)
  {
    // Ask user to save unsaved unsaved files
    if(saveButton.isEnabled())
    {
      int res =  JOptionPane.showConfirmDialog(this, "Save " + getTitle() + "?");
      if(res == JOptionPane.OK_OPTION)
      {
        saveButton_actionPerformed(null);
      }
    }

    theText.setText("");
    currentFileName = null;
    setTitle("<Untitled-" + par.parent.seed++ + ">");
    saveButton.setEnabled(false);
    saveButton.repaint();
    theText.repaint();
  }

 class MyTimer extends Thread
 {
   int m;

   public MyTimer(int minutes)
   {
     m = (minutes == 0) ? 1 : minutes;
   }

   public void run()
   {
      while(true)
      {
        try
        {
          sleep(m*60*1000);
        }
        catch(Exception e) {;}

        if(currentFileName != null)
        {
          saveButton_actionPerformed(null);
        }
      }
   }
 }

  void removeHighlightPopup_actionPerformed(ActionEvent e)
  {
    light(Color.white);
  }

  void theText_mouseClicked(MouseEvent e)
  {
    if(tmpHighlight != null)
    {
      dark(tmpHighlight);
      tmpHighlight = null;
    }
  }



}


