
/**
 * A BeanBoxFrame acts as the top-level "frame" that contains a BeanBox.
 * <p>
 * The BeanBoxFrame manages the frame's menubar and keeps track of which
 * bean currently has the focus.
 * <p>
 * Note that there is an asumption that there is only one BeanBoxFrame
 * per application, so various method/fields are static.
 */

package SuperEdit.RAD;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;
import com.sun.java.swing.*;

public class BeanBoxFrame extends JInternalFrame implements LayoutManager, Runnable, ActionListener {

    private static String tmpDir = "tmp";
    private static boolean doShowTimes = false;
    private static Thread focusThread;
    private static Component nextFocus;
    private static BeanBoxFrame instance;
    private static String clipDir = "tmp";
    private static String clipFile = "beanBoxClip.ser";
    private static String clipResource = "beanBoxClip";
    private static String versionID = "Hacked Beanbox";
    private static String clipLabel;
    private static String clipName;
    private static boolean clipFromPrototypeInfo;
    private static boolean quickStart = false;
    private static boolean defineOnDemand = true;
    public Frame PARENT_FRAME;

    public MenuFrame menuWindow = new MenuFrame(this);
    ImageIcon ico = loadImageIcon("images/logo.gif" , "Menu");

    /**
     * Create a new BeanBoxFrame at a default screen location.
     */

    public BeanBoxFrame(JDesktopPane desktop, Frame pf, boolean quickStart, boolean doShowTimes) {

	super("GUI Builder");
  PARENT_FRAME = pf;
  this.quickStart = quickStart;
  this.doShowTimes = doShowTimes;
  this.desktop = desktop;


	// there should only be one instance of this class.	
	if (instance != null) {
	    throw new Error("Attempt to create multiple instances of BeanBoxFrame");
	}
	instance = this;

	getContentPane().setLayout(new BorderLayout());
	setBackground(Color.lightGray);
	this.setMenuBar(new JMenuBar());

	// Timing note: the setFont causes AWT to initialiie itself,
	// so a large chunk of time happens here.
	setFont(new Font("Dialog", Font.PLAIN, 10));

        Timer tim = new Timer();
	toolBox = new ToolBox(20, 20);
  desktop.add(toolBox, JLayeredPane.PALETTE_LAYER);

    toolBox.setResizable(true);
    toolBox.setIconifiable(true);

  toolBox.setBounds(30, 30, 175, 500);

  desktop.add(menuWindow, JLayeredPane.DRAG_LAYER);
  menuWindow.setBounds(675, 220, 258, 190);
  menuWindow.setResizable(true);
  menuWindow.setFrameIcon(new ImageIcon(ico.getImage().getScaledInstance(18, 18, Image.SCALE_SMOOTH)));
  menuWindow.setIconifiable(true);
  menuWindow.setMaximizable(true);
  menuWindow.setTitle("Menu Window");

	if (showTimes()) {
	    System.err.println("new Toolbox => " + tim.elapsed());
	    tim.reset();
	}

	topBox = new BeanBox();
	topBox.setBackground(Color.lightGray);
	topWrapper = new Wrapper(topBox, "BeanBox", "sun.beanbox.BeanBox");
	topWrapper.setBackground(Color.lightGray);
	getContentPane().add(topWrapper, BorderLayout.CENTER);

	if (showTimes()) {
	    System.err.println("new BeanBox => " + tim.elapsed());
	    tim.reset();
	}

	doSetCurrentFocus(topWrapper);
	topBox.setSize(100,100);
	getContentPane().setLayout(this);

	setBounds(170, 20, 450, 500);
	//$new WindowCloser(this, true);
	show();

    	propertySheet = new PropertySheet(topWrapper, 575, 20, desktop);

      desktop.add(propertySheet, JLayeredPane.POPUP_LAYER);
      propertySheet.setLocation(675, 30);

    propertySheet.setResizable(true);
    propertySheet.setIconifiable(true);


	// Create a thread to handle focus changes.
	focusThread = new Thread(this);
	focusThread.start();
    }

    private boolean inBeanBox(Component c) {
	while (c != null) {
	    if (c instanceof BeanBox) {
		return true;
	    }
            c = c.getParent();
	}
	return false;
    }

    public void actionPerformed(ActionEvent evt) {
	// Menu actions come here,
	// We forward them to the current BeanBox.
        getCurrentBeanBox().queueMenuItem(evt);
    }

    static Object getCurrentBean() {
	return currentFocus.getBean();
    }

    static Component getCurrentComponent() {
	return currentFocus.getChild();
    }


    static Wrapper getCurrentWrapper() {
	return currentFocus;
    }

    static BeanBox getCurrentBeanBox() {
	Component c = getCurrentComponent();
	for (;;) {
	    if (c == null) {
	        System.err.println("No BeanBox in focus???");
		return null;
	    }
	    if (c instanceof BeanBox) {
		return (BeanBox) c;
	    }
	    c = c.getParent();
	}
    }


   static void setCurrentComponent(Component focus) {
	// Null means focus on the top-level beanbox.
	if (focus == null) {
	    focus = topWrapper;
	}
	// Wakeup our internal thread to do the focus change.
	synchronized (instance) {
	    nextFocus = focus;
	    instance.notifyAll();
	}	
    }

    // Internal thread to handle focus changes.  We use a separate
    // thread for this to offload work from the event thread in order
    // to avoid deadlocks.
    public void run() {
	for (;;) {
	    Component bean;
	    synchronized (this) {
		while (nextFocus == null) {
		    try {
		        wait();
		    } catch (Exception ex) {
			System.err.println("Unexpected interrupt in focus thread");
		    }
		}
		bean = nextFocus;
		nextFocus = null;
	    }
	    doSetCurrentFocus(bean);
	}
    }

    private void doSetCurrentFocus(Component focus) {
	Wrapper target = Wrapper.findWrapper(focus);

	if (target == currentFocus) {
	    return;
	}
	
	// Deactivate the old Wrapper.
	if (currentFocus != null) {
	    currentFocus.deactivate();
	}

	currentFocus = target;

        // Activate the new Wrapper.
	currentFocus.activate();

	// Find the nearest surrounding BeanBox and use its menubar.
	for (Component c = target.getChild(); c != null; c = c.getParent()) {
	    if (c instanceof BeanBox) {
		BeanBox hdr = (BeanBox) c;
		hdr.updateJMenuBar(getMenuBar());
		break;
	    }
	}

	// Point the property sheet at the new focus.
	if (propertySheet != null) {
	    propertySheet.setTarget(getCurrentWrapper());
	}
    }

    public void setCustomizer(Customizer c) {
	if (propertySheet != null) {
	    propertySheet.setCustomizer(c);
	}
    }

    public void dispose() {
	System.exit(0);
    }

    public static String getClipDir() {
	return clipDir;
    }

    public static String getClipFileName() {
	if (clipDir != null) {
	    return clipDir+java.io.File.separator+clipFile;
	} else {
	    return clipFile;
	}
    }

    // This method is currently unused
    public static String getClipResource() {
	return clipResource;
    }

    public static String getTmpDir() {
	return tmpDir;
    }

    public static void setClipName(String name) {
        clipName = name;
    }

    public static String getClipName() {
        return clipName;
    }
    
    // Added the following getter/setters.
    // We need to propogate the fromPrototype info across copies and pastes
    // since beans originating from .ser files ("prototypes") must be
    // effectivley treated as having hidden-state at code generation time.
    // If we didn't propogate this info, we would have to generate
    // initialization statements for all properties, not just those modified
    // by the user at code generation time.
    
    public static void setClipFromPrototypeInfo(boolean prototypeInfo) {
    	clipFromPrototypeInfo = prototypeInfo;
    }
    
    public static boolean getClipFromPrototypeInfo() {
    	return clipFromPrototypeInfo;
    }

    public static void setClipLabel(String label) {
        clipLabel = label;
    }

    public static String getClipLabel() {
        return clipLabel;
    }

    public static boolean getQuickStart() {
	return quickStart;
    }

    public static boolean getDefineOnDemand() {
	return defineOnDemand;
    }

    //----------------------------------------------------------------------
    // Layout related stuff.
    //----------------------------------------------------------------------

    public void addLayoutComponent(String name, Component comp) {
    }

    public void removeLayoutComponent(Component comp) {
    }

    public Dimension preferredLayoutSize(Container parent) {
	return new Dimension(400,300);
    }

    public Dimension minimumLayoutSize(Container parent) {
	return new Dimension(100,100);
    }

    public void layoutContainer(Container parent) {
	Dimension d = parent.getSize();
	Insets ins = parent.getInsets();
	int width = d.width - (ins.left + ins.right);
	int height = d.height - (ins.top + ins.bottom);
	topWrapper.setBounds(ins.left, ins.top, width, height);
	topWrapper.invalidate();
	topWrapper.doLayout();
    }

    public static BeanBox getTopBox() {
	return topBox;
    }

    public static PropertySheet getPropertySheet() {
	return propertySheet;
    }

    public static Wrapper getTopWrapper() {
	return topWrapper;
    }   

    public static ToolBox getToolBox() {
	return toolBox;
    }

    public static boolean showTimes() {
	return doShowTimes;
    }

    public static boolean getHideInvisibleBeans() {
	return hideInvisibleBeans;
    }

    public static void setHideInvisibleBeans(boolean hide) {
	hideInvisibleBeans = hide;
	Wrapper.showInvisibleBeans(!hide);
    }

    public static void setDesignTime(boolean designMode) {
	Beans.setDesignTime(designMode);
	if (designMode) {
	    toolBox.setVisible(true);
	    propertySheet.setVisible(true);
	} else {
	    toolBox.setVisible(false);
	    propertySheet.setVisible(false);
	}
    }

    public static String getVersionID() {
	return versionID;
    }

    //----------------------------------------------------------------------

    private static BeanBox topBox;
    private static Wrapper topWrapper;

    private static Wrapper currentFocus;

    static PropertySheet propertySheet;
    private static ToolBox toolBox;
    JDesktopPane desktop;

    private static boolean hideInvisibleBeans;

   public void setDesignable(boolean flag)
   {
     Beans.setDesignTime(flag);
   }

   JMenu M = new JMenu();
   JMenuBar MB = new JMenuBar();

   public JMenuBar getJMenuBar()
   {
     return MB;
   }

   public JMenuBar getMenuBar()
   {
     return MB;
   }

   public void setMenu(JMenu m)
   {
     M = m;
   }

   public void setJMenu(JMenu m)
   {
     M = m;
   }

   public void setMenuBar(JMenuBar mb)
   {
     MB = mb;
   }

   public void setJMenuBar(JMenuBar mb)
   {
     MB = mb;
   }

  public ImageIcon loadImageIcon(String filename, String description)
  {
      java.net.URL url = getClass().getResource(filename);
      return new ImageIcon(url, description);
  }
   

}
