
package SuperEdit.RAD;

import java.beans.*;
import java.lang.reflect.*;
import java.awt.*;
import java.util.Hashtable;
import java.util.Vector;
import com.sun.java.swing.*;
import borland.jbcl.layout.XYConstraints;
import borland.jbcl.layout.XYLayout;
import com.sun.java.swing.border.*;


public class PropertySheet extends JInternalFrame
{
  PropertySheetPanel panel;
  private boolean started;
  JDesktopPane desktop;
  PropertySheet mainFrame = this;

  PropertySheet(Wrapper target, int x, int y, JDesktopPane desktop)
  {
	  super("Properties - <initializing...>");
    setResizable(true);
    PropertyEditorManager.registerEditor(java.awt.Dimension.class, SuperEdit.RAD.DimensionEditor.class);
    //PropertyEditorManager.registerEditor(Border.class, BorderEditor.class);
	  getContentPane().setLayout(new BorderLayout());

	  setBounds(x,y, 100, 100);

	  panel = new PropertySheetPanel();
    getContentPane().add(new JScrollPane(panel), BorderLayout.CENTER);

	  panel.setTarget(target);
	  setTitle("Properties - " + (target.getBeanLabel().equalsIgnoreCase("BeanBox") ? "Gui Builder" : target.getBeanLabel()));

	  started = true;
    setVisible(true);
  }

  void setTarget(Wrapper targ)
  {
    setVisible(false);
	  Object bean = targ.getBean();
	  String displayName = targ.getBeanLabel();
	  panel.setTarget(targ);
	  setTitle("Properties - " + displayName);
    setVisible(true);
  }

  void setCustomizer(Customizer c)
  {
    panel.setCustomizer(c);
  }

  void wasModified(PropertyChangeEvent evt)
  {
    panel.wasModified(evt);
  }

// ****************** START PANEL INNER CLASS **************************

class PropertySheetPanel extends JPanel
{
    int yPos = 4;
    public boolean writable = true;

    int maxViewWidth = 150;
    int maxLabelWidth = 75;
    int visibleCount = 0;
    int frameHeight = 4;

  PropertySheetPanel()
  {
	  setLayout(new XYLayout());
  }

  synchronized void setTarget(Wrapper targ)
  {
	  removeAll();

	  if (target != null)
    {
	    setVisible(false);
    }

	  targetWrapper = targ;
	  target = targ.getBean();

    try
    {
	    BeanInfo bi = Introspector.getBeanInfo(target.getClass());
	    properties = bi.getPropertyDescriptors();
    }
    catch (IntrospectionException ex)
    {
	    error("PropertySheet: Couldn't introspect", ex);
	    return;
    }

	  editors = new PropertyEditor[properties.length];
	  values = new Object[properties.length];
	  views = new Component[properties.length];
	  labels = new JLabel[properties.length];

	  // Create an event adaptor.
	  EditedAdaptor adaptor = new EditedAdaptor(mainFrame);

	  for (int i = 0; i < properties.length; i++)
    {
	    // Don't display hidden or expert properties.
	    if (properties[i].isHidden() || properties[i].isExpert())
      {
		    continue;
	    }

	    String name = properties[i].getDisplayName();
	    Class type = properties[i].getPropertyType();
	    Method getter = properties[i].getReadMethod();
	    Method setter = properties[i].getWriteMethod();

	    // Only display read/write properties.
	    if (getter == null || setter == null)
      {
		    continue;
	    }

	    if (setter == null)
      {
		    writable = false;
	    }


	    Component view = null;

	    try
      {
		    Object args[] = { };
		    Object value = getter.invoke(target, args);
        values[i] = value;

        PropertyEditor editor = null;
        Class pec = properties[i].getPropertyEditorClass();
		    if (pec != null)
        {
		      try
          {
			      editor = (PropertyEditor)pec.newInstance();
          }
          catch (Exception ex)
          {
			      // Drop through.
          }
        }
		    if (editor == null)
        {
          //if(type.isInterface())
          //{
          //  editor = getInterfaceEditor(type);
          //}
          //else
          //{
		        editor = PropertyEditorManager.findEditor(type);
          //}
        }
        editors[i] = editor;

        // If we can't edit this component, skip it.

        if(editor == null)
        {
          editor = new DefaultEditor();
          writable = false;
        }

        editor.setValue(value);
        editor.addPropertyChangeListener(adaptor);

		    // Now figure out how to display it...
		    if (editor.isPaintable() && editor.supportsCustomEditor())
        {
		      view = new PropertyCanvas(mainFrame, editor);
          if(!writable) {view.setEnabled(false); writable = true;}
        }
        else if (editor.getTags() != null)
        {
		      view = new PropertySelector(editor);
          if(!writable) {view.setEnabled(false); writable = true;}
        }
        else if (editor.getAsText() != null)
        {
		      String init = editor.getAsText();
		      view = new PropertyText(editor);
          if(!writable) {view.setEnabled(false); writable = true;}
        }
        else
        {
		      System.err.println("Warning: Property \"" + name
				  + "\" has non-displayabale editor.  Skipping.");
		      continue;
        }

      }
      catch (InvocationTargetException ex)
      {
		    System.err.println("Skipping property " + name + " ; exception on target: " + ex.getTargetException());
		    ex.getTargetException().printStackTrace();
		    continue;
      }
      catch (Exception ex)
      {
		    System.err.println("Skipping property " + name + " ; exception: " + ex);
		    ex.printStackTrace();
		    continue;
	    }

	    labels[i] = new JLabel(name, JLabel.RIGHT);
	    //add(labels[i]);

	    views[i] = view;
	    //add(views[i]);
    }

    // Add components and do my custom layout

    maxViewWidth = 150;
    maxLabelWidth = 75;
    visibleCount = 0;
    frameHeight = 4;


    for(int z = 0; z < labels.length; z++)
    {
      if(labels[z] != null)
      {
        maxLabelWidth = Math.max(labels[z].getPreferredSize().width, maxLabelWidth);
        maxViewWidth = Math.max(views[z].getPreferredSize().width, maxViewWidth);

        int ht = Math.max(labels[z].getPreferredSize().height, views[z].getPreferredSize().height);
        ht = ht > 20 ? ht : 20;

        frameHeight =  frameHeight + 4 + ht;
        visibleCount += 1;
      }
    }


    for(int z = 0; z < labels.length; z++)
    {
      if(labels[z] != null)
      {
        int ht = Math.max(labels[z].getPreferredSize().height, views[z].getPreferredSize().height);

        ht = ht > 20 ? ht : 20;

        add(labels[z], new XYConstraints(4, yPos, maxLabelWidth, ht));
        add(views[z], new XYConstraints(4 + 10 + maxLabelWidth, yPos, maxViewWidth, ht));

        yPos = yPos + ht + 4;
      }
    }

    yPos = 4;

    // **************************************

	  processEvents = true;
	  doLayout(true);
	  setVisible(true);
  }

  /*void stretch()
  {
	  // This gets called when a user explicitly resizes the frame.

	  Component child = null;
	  try
    {
	    child = frame.getComponent(0);
    }
    catch (Exception ex)
    {
	    // frame has no active children;
	    return;
    }
	  Dimension childSize = child.getSize();
	  Dimension frameSize = frame.getSize();
	  Insets ins = frame.getInsets();
	  int vpad = ins.top + ins.bottom;
	  int hpad = ins.left + ins.right;

	  // If the frame size hasn't changed, do nothing.
	  if (frameSize.width == (childSize.width + hpad) &&
		frameSize.height == (childSize.height + vpad))
    {
	    return;
    }

	  // We treat the new frame sizes as a future maximum for our own
	  // voluntary size changes.
	  maxHeight = frameSize.height;
	  maxWidth = frameSize.width;

	  // If we've gotten smaller, force new layout.
	  if (frameSize.width < (childSize.width + hpad) ||
			frameSize.height < (childSize.height + vpad))
    {
	    // frame has shrunk in at least one dimension.
	    setTarget(targetWrapper);
    }
    else
    {
	   // Simply resize the contents.  Note that this won't make
	   // any ScrollPane go away, that will happen on the next
	   // focus change.
 	   child.setSize(frameSize.width - hpad, frameSize.height - vpad);
    }
  }
  */

  private void doLayout(boolean doSetSize)
  {
	  if (views == null || labels == null)
    {
	    return;
    }

	  if (doSetSize)
    {
	    mainFrame.setSize(maxLabelWidth + maxViewWidth + 4 + 10 + 4 + 15, frameHeight + 40);
    }

    //this.getLayout().layoutContainer(this);
    super.doLayout();

  }

  public PropertyEditor getInterfaceEditor(Class type)
  {
    if(type.getName().endsWith("Border"))
    {
      return new BorderEditor();
    }
    else
    {
      return PropertyEditorManager.findEditor(type);
    }
  }


  public void doLayout()
  {
	  doLayout(false);
  }


  synchronized void setCustomizer(Customizer c)
  {
	  if (c != null)
    {
	    c.addPropertyChangeListener(new EditedAdaptor(mainFrame));
    }
  }

  synchronized void wasModified(PropertyChangeEvent evt)
  {
	  if (!processEvents)
    {
	    return;
    }

	  if (evt.getSource() instanceof PropertyEditor)
    {
	    PropertyEditor editor = (PropertyEditor) evt.getSource();
	    for (int i = 0 ; i < editors.length; i++)
      {
        if (editors[i] == editor)
        {
		      PropertyDescriptor property = properties[i];
		      Object value = editor.getValue();
		      values[i] = value;
		      Method setter = property.getWriteMethod();
		      try
          {
		        Object args[] = { value };
		        args[0] = value;
		        setter.invoke(target, args);

            // We add the changed property to the targets wrapper
            // so that we know precisely what bean properties have
            // changed for the target bean and we're able to
            // generate initialization statements for only those
            // modified properties at code generation time.
            targetWrapper.getChangedProperties().addElement(properties[i]);

          }
          catch (InvocationTargetException ex)
          {
		        if (ex.getTargetException()
		                instanceof PropertyVetoException)
            {
			        warning("Vetoed; reason is: "
			        + ex.getTargetException().getMessage());
            }
		        else
		          error("InvocationTargetException while updating "
              + property.getName(), ex.getTargetException());
          }
          catch (Exception ex)
          {
		        error("Unexpected exception while updating "
            + property.getName(), ex);
          }
          if (views[i] != null && views[i] instanceof PropertyCanvas)
          {
		        views[i].repaint();
          }
		      break;
        }
      }
    }

	  // Now re-read all the properties and update the editors
	  // for any other properties that have changed.
	  for (int i = 0; i < properties.length; i++)
    {
	    Object o;
	    try
      {
        Method getter = properties[i].getReadMethod();
        Object args[] = { };
        o = getter.invoke(target, args);
	    }
      catch (Exception ex)
      {
		    o = null;
	    }
	    if (o == values[i] || (o != null && o.equals(values[i])))
      {
        // The property is equal to its old value.
		    continue;
	    }
	    values[i] = o;
	    // Make sure we have an editor for this property...
	    if (editors[i] == null)
      {
		    continue;
	    }
	    // The property has changed!  Update the editor.
	    editors[i].setValue(o);
	    if (views[i] != null)
      {
		    views[i].repaint();
	    }
    }

	  // Make sure the target bean gets repainted.
	  if (Beans.isInstanceOf(target, Component.class))
    {
	    ((Component)(Beans.getInstanceOf(target, Component.class))).repaint();
    }
  }

  private void warning(String s)
  {
	  new ErrorDialog(null, "Warning: " + s);
  }

  //----------------------------------------------------------------------
  // Log an error.

  private void error(String message, Throwable th)
  {
	  String mess = message + ":\n" + th;
	  System.err.println(message);
	  th.printStackTrace();
	  // Popup an ErrorDialog with the given error message.
	  new ErrorDialog(null, mess);

  }

  //----------------------------------------------------------------------

  // We need to cache the targets' wrapper so we can annoate it with
  // information about what target properties have changed during design
  // time.

  class DefaultEditor extends PropertyEditorSupport
  {

  }
}

// ***************** END PANEL INNER CLASS **************************

  private Wrapper targetWrapper;
  private Object target;
  private PropertyDescriptor properties[];
  private PropertyEditor editors[];
  private Object values[];
  private Component views[];
  private JLabel labels[];

  private boolean processEvents;
  private static int hPad = 4;
  private static int vPad = 4;
  private int maxHeight = 500;
  private int maxWidth = 300;

}


