// An operand for a command.

/*
 * Copyright 1997 by John D. Ramsdell
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package edu.neu.ccs.jmk;

/**
 * An operand for a command.
 * An operand is a string or one of the four command variables.
 * The command variables are represented by the characters,
 * '@', '&lt;', '?', or '%'.
 * @see edu.neu.ccs.jmk.Command
 * @version November 1997
 * @author John D. Ramsdell
 */
final class Operand
{
  private String string;
  // The operand is a command variable if ch is one of
  // '@', '<', '?', or '%'.
  // Otherwise, the operand is a string operand.
  private char ch = 0;

  /**
   * Create a string operand.
   */
  Operand(String string) {
    this.string = string;
  }

  /**
   * Create a command variable operand if argument is one of
   * '@', '&lt;', '?', or '%'.   Otherwise, create a string operand
   * with a null value.
   */
  Operand(char ch) {
    this.ch = ch;
  }

  /**
   * Get a character which identifies the object as a command variable
   * operand or a string operand.
   */
  char getChar() {
    return ch;
  }

  /**
   * Get the string associated with a string operand.
   */
  String getString() {
    return string;
  }
}
